#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <confdb.h>
#include <fviews.h>
#include <subsys.h>
#include <configf.h>
#include "../../paths.h"
#include "redhat.h"

static HELP_FILE help_clock ("redhat","clock");
static HELP_FILE help_keyboard ("redhat","keyboard");
static HELP_FILE help_network ("redhat","network");
static HELP_FILE help_sendmail ("redhat","sendmail");
static HELP_FILE help_sysctl ("redhat","sysctl");
static HELP_FILE help_desktop ("redhat","desktop");

static CONFIG_FILE f_clock("/etc/sysconfig/clock",help_clock
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0644);

static CONFIG_FILE f_keyboard("/etc/sysconfig/keyboard",help_keyboard
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0644);

static CONFIG_FILE f_network("/etc/sysconfig/network",help_network
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0644,subsys_netclient);

static CONFIG_FILE f_routed("/etc/sysconfig/routed",help_network
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0644,subsys_netclient);

static CONFIG_FILE f_sendmail("/etc/sysconfig/sendmail",help_sendmail
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0644,subsys_netclient);

static CONFIG_FILE f_sysctl (ETC_SYSCTL_CONF,help_sysctl
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0644,subsys_netclient);

static CONFIG_FILE f_desktop ("/etc/sysconfig/desktop",help_desktop
	,CONFIGF_MANAGED|CONFIGF_OPTIONNAL
	,"root","root",0644);

enum REDHAT_FILE { RH_clock,RH_keyboard,RH_network,RH_routed,
	RH_sendmail, RH_sysctl, RH_dummy };

enum REDHAT_TYPE {REDHAT_STR,REDHAT_YESNO,REDHAT_ONOFF,REDHAT_VAL0, REDHAT_BOOL01};

struct RHLK{
	const char *key1;
	const char *key2;
	const char *var;
	REDHAT_FILE file;
	REDHAT_TYPE type;	
};

static RHLK tblk[]={
	{"linuxconf",	"keymap",		"KEYTABLE",		RH_keyboard,	REDHAT_STR},
	{"datetime",	"universal",	"UTC",			RH_clock,		REDHAT_YESNO},
	{"datetime",	"alphaarc",		"ARC",			RH_clock,		REDHAT_YESNO},
	{"nis",			"domain",		"NISDOMAIN",	RH_network,		REDHAT_STR},
	{"routing",		"gateway",		"GATEWAY",		RH_network,		REDHAT_STR},
	{"routing",		"gatewaydev",	"GATEWAYDEV",	RH_network,		REDHAT_STR},
	// Linuxconf maintains both the old and new IP forwarding
	// The first one is used to read, both are used to update
	// The CONFDB_REDHAT constructor disables the first one if
	// /etc/sysctl.conf does not exist (i.e. on redhat releases
	// prior to 6.2)
	{"routing",		"ipv4",			"net.ipv4.ip_forward",	RH_sysctl,		REDHAT_BOOL01},
	{"routing",		"ipv4",			"FORWARD_IPV4",	RH_network,		REDHAT_YESNO},
	{"routed",		"gateway",		"EXPORT_GATEWAY",RH_routed,		REDHAT_YESNO},
	{"routed",		"silent",		"SILENT",		RH_routed,		REDHAT_YESNO},

	{"ipx",			"active",		"IPX",			RH_network,		REDHAT_YESNO},
	{"ipx",			"internal_netnum",
									"IPXINTERNALNETNUM",		RH_network,		REDHAT_STR},
	{"ipx",			"internal_nodenum",
									"IPXINTERNALNODENUM",		RH_network,		REDHAT_STR},
	{"ipx",			"primary_auto",	"IPXAUTOPRIMARY",	RH_network,		REDHAT_ONOFF},
	{"ipx",			"frame_auto",	"IPXAUTOFRAME",		RH_network,		REDHAT_ONOFF},

	{"mailconf",	"queuedelay",	"QUEUE",			RH_sendmail,	REDHAT_VAL0},
	/* Special dummy entry. We are using pseudo entry in conf.linuxconf */
	/* to handle distribution "preferences" */
	{"--distrib--",	"sysvrestart",	NULL,			RH_dummy,		REDHAT_STR},
	{"--distrib--",	"privgroup",	NULL,			RH_dummy,		REDHAT_STR},
	{"--distrib--",	"NETWORKING",	"NETWORKING",	RH_network,		REDHAT_YESNO},
	{"--distrib--",	"HOSTNAME",		"HOSTNAME",		RH_network,		REDHAT_STR},
	{"--distrib--",	"sysvenh",		NULL,			RH_dummy,		REDHAT_STR},
	{"--distrib--",	"rpmvendor",	NULL,			RH_dummy,		REDHAT_STR},
	{"--distrib--",	"rpmdistribution",	NULL,		RH_dummy,		REDHAT_STR},
	{"--distrib--",	"release",		NULL,		RH_dummy,		REDHAT_STR},
	{"--distrib--",	"desktop",		"DESKTOP",		RH_dummy,		REDHAT_STR},
};
#define NBLK (sizeof(tblk)/sizeof(tblk[0]))


class CONFDB_REDHAT: public CONFDB{
	CONFDB *orig;	// Original CONFDB object managing conf.linuxconf
	struct {
		VIEWITEMS *clock;
		VIEWITEMS *keyboard;
		VIEWITEMS *network;
		VIEWITEMS *routed;
		VIEWITEMS *sendmail;
		VIEWITEMS *sysctl;
		VIEWITEMS *desktop;
		bool clock_modified,keyboard_modified;
		bool network_modified,routed_modified;
		bool sendmail_modified;
		bool sysctl_modified;
		bool desktop_modified;
	} sys;
	SSTRINGS tbstr;		// Old temporary strings returned
						// to caller of getval
	/*~PROTOBEG~ CONFDB_REDHAT */
public:
	CONFDB_REDHAT (CONFDB *_orig);
	void add (const char *prefix,
		 const char *key,
		 const char *val);
	int archive (SSTREAM&ss, const char *_sys);
	int extract (SSTREAM&ss, const char *_sys);
	int getall (const char *prefix,
		 const char *key,
		 SSTRINGS&lst,
		 bool copy);
	const char *getval (const char *prefix,
		 const char *key,
		 const char *defval);
private:
	const char *getvalfromrh (RHLK *rh);
public:
	void removeall (const char *prefix,
		 const char *key);
private:
	void reset_modified (void);
public:
	int save (PRIVILEGE *priv);
	void setcursys (const char *_subsys);
private:
	void updaterh (RHLK *rh,
		 const char *key,
		 const char *val);
public:
	~CONFDB_REDHAT (void);
	/*~PROTOEND~ CONFDB_REDHAT */
};

PRIVATE void CONFDB_REDHAT::reset_modified()
{
	sys.keyboard_modified = false;
	sys.clock_modified = false;
	sys.network_modified = false;
	sys.routed_modified = false;
	sys.sendmail_modified = false;
	sys.sysctl_modified = false;
	sys.desktop_modified = false;
}

static RHLK *filter_lookup (
	const char *key1,
	const char *key2,
	unsigned &start)		// Start here in the search
{
	RHLK *ret = NULL;
	RHLK *pt = tblk+start;
	for (unsigned i=start; i<NBLK; i++, pt++){
		if (strcmp(pt->key1,key1)==0
			&& strcmp(pt->key2,key2)==0){
			ret = pt;
			start = i+1;
			break;
		}
	}
	return ret;
}

PUBLIC CONFDB_REDHAT::CONFDB_REDHAT(CONFDB *_orig)
{
	orig = _orig;
	sys.clock = NULL;
	sys.keyboard = NULL;
	sys.network = NULL;
	sys.routed = NULL;
	sys.sendmail = NULL;
	sys.sysctl = NULL;
	sys.desktop = NULL;
	reset_modified();
	if (!f_sysctl.exist()){
		unsigned start = 0;
		RHLK *rh = filter_lookup ("routing","ipv4",start);
		if (rh != NULL){
			rh->key1 = "?????";	// Disable the entry
		}
	}
}

PUBLIC CONFDB_REDHAT::~CONFDB_REDHAT()
{
	delete orig;
	delete sys.clock;
	delete sys.keyboard;
	delete sys.network;
	delete sys.routed;
	delete sys.sendmail;
	delete sys.sysctl;
	delete sys.desktop;
}

PUBLIC void CONFDB_REDHAT::setcursys (const char *_subsys)
{
	orig->setcursys (_subsys);
}

static void saveconf (
	VIEWITEMS *&items,
	CONFIG_FILE &cf,
	PRIVILEGE *priv,
	bool modified)
{
	if (items != NULL){
		if (modified) items->write (cf,priv);
		delete items;
		items = NULL;
	}
}


PUBLIC int CONFDB_REDHAT::save(PRIVILEGE *priv)
{
	int ret = orig->save(priv);
	if (ret != -1){
		saveconf (sys.clock,f_clock,priv,sys.clock_modified);
		saveconf (sys.keyboard,f_keyboard,priv,sys.keyboard_modified);
		saveconf (sys.network,f_network,priv,sys.network_modified);
		saveconf (sys.routed,f_routed,priv,sys.routed_modified);
		saveconf (sys.sendmail,f_sendmail,priv,sys.sendmail_modified);
		saveconf (sys.sysctl,f_sysctl,priv,sys.sysctl_modified);
		saveconf (sys.desktop,f_desktop,priv,sys.desktop_modified);
		reset_modified();
	}
	tbstr.remove_all();
	return ret;
}

/*
	Return the release of the redhat distribution
	release 4.2 becomes 402
*/

static CONFIG_FILE f_release("/etc/redhat-release",help_nil
	,CONFIGF_PROBED|CONFIGF_NOARCH|CONFIGF_OPTIONAL);
static CONFIG_FILE f_mdkrelease("/etc/mandrake-release",help_nil
	,CONFIGF_PROBED|CONFIGF_NOARCH|CONFIGF_OPTIONAL);

static int redhat_readrelease()
{
	int ret = 501;		// Assume a redhat 5.1 default
						// for redhat based distribution without
						// an /etc/redhat-release file
						// redhat 5.1 is the first linuxconf aware redhat
	FILE_CFG *fin = f_release.fopen ("r");
	if (fin != NULL){
		char line[200];
		while (fgets (line,sizeof(line)-1,fin) != NULL){
			const char *pt = strstr (line,"Raw Hide");
			if (pt != NULL){
				ret = 700;
				break;
			}else {
				pt = strstr (line,"release");
				if (pt != NULL){
					const char *ver = str_skip (pt+7);
					int num1 = atoi(ver);
					int num2 = 0;
					const char *pt = strchr(ver,'.');
					if (pt != NULL) num2 = atoi(pt+1);
					ret = num1*100 + num2;
					break;
				}
			}
		}
		fclose (fin);
	}
	return ret;
}

/*
	Return the redhat release in format XYY (5.2 -> 502)
*/
int redhat_release()
{
	static int ret = -1;	// Read the file only once
	if (ret == -1){
		ret = redhat_readrelease();
		if (redhat_ismdk()){
			/* #Specification: redhat release / mandrake
				So far (as of mandrake 7.1), RedHat and Mandrake are
				still fairly the same. At some point, we may have to
				introduce a mandrake module and some mandrake
				specific resource files. For now, we cheat a bit.
				RedHat 7.x will is using some different configuration
				files (/etc/modules.conf instead of /etc/conf.modules)
				so needs some specific resource file. Unfortunatly
				mandrake 7.x is already out and is closer to RedHat 6.2.

				So for now, we map mandrake 7.0, 7.1 and 7.2 to redhat 6.2.
			*/
			if (ret == 700 || ret == 701 || ret == 702){
				ret = 602;
			}else if (ret >= 800){
				// Another trickery. Release goes like 7.0, 7.1, 7.2 and
				// never reach 7.10. So we jump mandrake release to 8.10
				// so they won't clash with redhat 8.0,1,2.
				// Still a trick, but so far, the distrib are still so close.
				ret += 10;
			}
		}
	}
	return ret;
}

/*
	Is this a mandrake distribution ?
*/
bool redhat_ismdk()
{
	return f_mdkrelease.exist();
}

/*
	Read the Mandrake release.
	In most of this module, we fake mandrake as one redhat release
	but sometime we have to know for sure.
*/
int redhat_mdkrelease()
{
	return redhat_readrelease();
}


PRIVATE const char *CONFDB_REDHAT::getvalfromrh(RHLK *rh)
{
	const char *ret = NULL;
	VIEWITEMS **rhf = NULL;
	CONFIG_FILE *cf = NULL;
	if (rh->file == RH_dummy){
		int release = redhat_release();
		if (strcmp(rh->key2,"sysvrestart")==0){
			ret = release >= 500 ? "1" : "0";
		}else if (strcmp(rh->key2,"privgroup")==0){
			ret = "1";
		}else if (strcmp(rh->key2,"sysvenh")==0){
			ret = release > 500 ? "1" : "0";
		}else if (strcmp(rh->key2,"rpmvendor")==0){
			ret = vendor_getid();
		}else if (strcmp(rh->key2,"rpmdistribution")==0){
			ret = vendor_getdistid();
		}else if (strcmp(rh->key2,"release")==0){
			static char release_str[10];
			sprintf (release_str,"%d.%d",release/100,release%100);
			ret = release_str;
		}else if (strcmp(rh->key2,"desktop")==0){
			if (release >= 700){
				rhf = &sys.desktop;
				cf = &f_desktop;
			}else{
				static char desktop_str[20];
				desktop_str[0] = '\0';
				FILE_CFG *fin = f_desktop.fopen ("r");
				if (fin != NULL){
					fscanf (fin,"%s\n",desktop_str);
					fclose (fin);
				}
				ret = desktop_str;
			}
		}
	}else if (rh->file == RH_clock){
		rhf = &sys.clock;
		cf = &f_clock;
	}else if (rh->file == RH_keyboard){
		rhf = &sys.keyboard;
		cf = &f_keyboard;
	}else if (rh->file == RH_network){
		rhf = &sys.network;
		cf = &f_network;
	}else if (rh->file == RH_routed){
		rhf = &sys.routed;
		cf = &f_routed;
	}else if (rh->file == RH_sendmail){
		rhf = &sys.sendmail;
		cf = &f_sendmail;
	}else if (rh->file == RH_sysctl){
		rhf = &sys.sysctl;
		cf = &f_sysctl;
	}
	if (rhf != NULL){
		if (*rhf == NULL){
			*rhf = new VIEWITEMS;
			(*rhf)->read (*cf);
		}
		char tmp[1000];
		const char *val = (**rhf).locateval (rh->var,tmp);
		if (val != NULL){
			if (rh->type == REDHAT_YESNO
				|| rh->type == REDHAT_ONOFF
				|| rh->type == REDHAT_BOOL01){
				ret = stricmp(val,"yes")==0
					|| stricmp(val,"true")==0
					|| stricmp(val,"on")==0
					|| stricmp(val,"1")==0
					? "1" : "0";
			}else if (rh->type == REDHAT_VAL0 && val[0] == '\0'){
				ret = "0";
			}else{
				SSTRING *n = new SSTRING (val);
				tbstr.add (n);
				ret = n->get();
			}
		}
	}
	return ret;
}		

PRIVATE void CONFDB_REDHAT::updaterh(RHLK *rh, const char *key, const char *val)
{
	VIEWITEMS **rhf = NULL;
	CONFIG_FILE *cf = NULL;
	const char *quote = "\"";
	if (rh->type == REDHAT_YESNO){
		val = strcmp(val,"1")==0 ? "yes" : "no";
	}else if (rh->type == REDHAT_ONOFF){
		val = strcmp(val,"1")==0 ? "on" : "off";
	}else if (rh->type == REDHAT_VAL0){
		if (strcmp(val,"0")==0) val = "";
	}else if (rh->type == REDHAT_BOOL01){
		quote = "";
	}
	if (rh->file == RH_clock){
		rhf = &sys.clock;
		cf = &f_clock;
		sys.clock_modified = true;
	}else if (rh->file == RH_keyboard){
		rhf = &sys.keyboard;
		cf = &f_keyboard;
		sys.keyboard_modified = true;
	}else if (rh->file == RH_network){
		rhf = &sys.network;
		cf = &f_network;
		sys.network_modified = true;
	}else if (rh->file == RH_routed){
		rhf = &sys.routed;
		cf = &f_routed;
		sys.routed_modified = true;
	}else if (rh->file == RH_sendmail){
		rhf = &sys.sendmail;
		cf = &f_sendmail;
		sys.sendmail_modified = true;
	}else if (rh->file == RH_sysctl){
		rhf = &sys.sysctl;
		cf = &f_sysctl;
		sys.sysctl_modified = true;
	}else if (rh->file == RH_dummy){
		if (strcmp(key,"desktop")==0){
			if (redhat_release()>=700){
				rhf = &sys.desktop;
				cf = &f_desktop;
				sys.desktop_modified = true;
			}else{
				FILE_CFG *fout = f_desktop.fopen ("w");
				if (fout != NULL){
					fprintf (fout,"%s\n",val);
					fclose (fout);
				}
			}
		}
	}
	if (rhf != NULL){
		if (*rhf == NULL){
			*rhf = new VIEWITEMS;
			(*rhf)->read (*cf);
		}
		char newline[strlen(rh->var)+1+1+strlen(val)+1+1];
		sprintf (newline,"%s=%s%s%s",rh->var,quote,val,quote);
		VIEWITEM *it = (*rhf)->locateassign (rh->var);
		if (it == NULL){
			it = new VIEWITEM (newline);
			(*rhf)->add (it);
		}else{
			it->line.setfrom (newline);
		}
	}
}		
	

PUBLIC const char *CONFDB_REDHAT::getval(
	const char *prefix,
	const char *key,
	const char *defval)
{
	unsigned start = 0;
	RHLK *rh = filter_lookup (prefix,key,start);
	const char *ret = defval;
	if (rh != NULL){
		/* #Specification: redhat files and conf.linuxconf / strategy
			A migration of some stuff stored in /etc/conf.linuxconf
			is done. The goal is to use from now on the redhat specific
			config file (in /etc/sysconfig) directly. A second goal
			is to let linuxconf users migrate to this strategy painlessly.

			To achieve that, we will use a strategy giving priority
			to the content of /etc/conf.linuxconf. Normally
			for all new linuxconf installation, /etc/conf.linuxconf
			won't have anything conflicting with the sysconfig file, so
			giving priority to it means nothing. For old linuxconf
			installation, conf.linuxconf does contain the more accurate
			(more recent) data.

			So the strategy is for reading

			#
			-read in conf.linuxconf first.
			-if there is something there, use it
			-if there is nothing, read the appropriate sysconfig file
			#

			And for writing

			#
			-erase all relevant entries in conf.linuxconf
			-update the appropriate sysconfig file
			#

			This strategy should provide a completly transparent
			migration.
		*/
		ret = orig->getval (prefix,key,NULL);
		if (ret == NULL){
			ret = getvalfromrh (rh);
			if (ret == NULL) ret = defval;
		}
	}else{
		ret = orig->getval (prefix,key,defval);
	}
	return ret;
}

PUBLIC int CONFDB_REDHAT::getall (
	const char *prefix,
	const char *key,
	SSTRINGS &lst,
	bool copy)	// Take a copy of the values
{
	return orig->getall (prefix,key,lst,copy);
}

PUBLIC void CONFDB_REDHAT::removeall (const char *prefix, const char *key)
{
	orig->removeall (prefix,key);
}
PUBLIC void CONFDB_REDHAT::add (
	const char *prefix,
	const char *key,
	const char *val)
{
	/* #specification: Cleaning conf.linuxconf / strategy
		All updates in conf.linuxconf are done using the same
		sequence

		#
		removeall
		add
		#

		The "removeall"s in the RedHat conf.linuxconf filter are going
		through normally. The "add"s are filtered. Some special entries
		are rerouted to the RedHat sysconfig files.

		This strategy, combined with the one for the getval() function
		provide the following functionnality:

		#
		-conf.linuxconf has priority to provide information
		-conf.linuxconf lose information at the first upgrade of this
		 information as the sysconfig file are updated
		#
	*/
	bool found = false;
	unsigned pos = 0;
	while (1){
		RHLK *rh = filter_lookup (prefix,key,pos);
		if (rh != NULL){
			updaterh (rh,key,val);
			found = true;
		}else{
			break;
		}
	}
	if (!found)	orig->add (prefix,key,val);
}

PUBLIC int CONFDB_REDHAT::archive (SSTREAM &ss, const char *_sys)
{
	return orig->archive (ss,_sys);
}


PUBLIC int CONFDB_REDHAT::extract (SSTREAM &ss, const char *_sys)
{
	return orig->extract (ss,_sys);
}

CONFDB *filter_fctnew (CONFDB *orig)
{
	return new CONFDB_REDHAT(orig);
}

/*
	Obtain a value from /etc/sysconfig/network
*/
static void filter_getval(const char *var, char *val)
{
	val[0] = '\0';
	VIEWITEMS items;
	if (items.read (f_network)!=-1){
		char tmp[1000];
		const char *pt = items.locateval (var,tmp);
		if (pt != NULL) strcpy (val,pt);
	}
}

/*
	Set the Host name in /etc/sysconfig/network
*/
int filter_setval(const char *var, const char *val)
{
	int ret = -1;
	VIEWITEMS items;
	if (items.read (f_network)!=-1){
		items.update (var,val);
		ret = items.write (f_network,NULL);
	}
	return ret;
}


static const char K_HOSTNAME[]="HOSTNAME";
/*
	Obtain the Host name from /etc/sysconfig/network
*/
void filter_gethostname(char *name)
{
	filter_getval (K_HOSTNAME,name);
}

/*
	Set the Host name in /etc/sysconfig/network
*/
int filter_sethostname(const char *name)
{
	return filter_setval (K_HOSTNAME,name);
}

#if 0
static const char K_GATEWAY[]="GATEWAY";
static const char K_GATEWAYDEV[]="GATEWAYDEV";
/*
	Obtain the Default gateway from /etc/sysconfig/network
*/
void filter_getgateway (char *gtw)
{
	filter_getval (K_GATEWAY,gtw);
}

/*
	Set the default gateway in /etc/sysconfig/network
*/
int filter_setgateway(const char *gtw, const char *device)
{
	return filter_setval (K_GATEWAY,gtw)
		| filter_setval (K_GATEWAYDEV,device);
}
#endif

